@@ -1,3 +1,5 @@
+#!/usr/bin/env python3
+
 """
 database.py
 
@@ -16,7 +18,7 @@ from graphviz import Digraph
 
 @lru_cache(maxsize=None)
 def read_sql(sql_file):
-    with open(pathlib.Path(__file__).parent.resolve() / "sql" / sql_file) as f:
+    with open(pathlib.Path.cwd() / ".." / "sql" / sql_file) as f:
         return f.read()
 
 
@@ -60,18 +62,29 @@ def add_nodes(nodes, ids):
     return _add_nodes
 
 
+def _upsert_node(cursor, identifier, data):
+    current_data = find_node(identifier)(cursor)
+    if not current_data:
+        # no prior record exists, so regular insert
+        _insert_node(cursor, identifier, data)
+    else:
+        # merge the current and new data and update
+        updated_data = {**current_data, **data}
+        cursor.execute(read_sql(
+            'update-node.sql'), (json.dumps(_set_id(identifier, updated_data)), identifier,))
+
+
 def upsert_node(identifier, data):
-    def _upsert_node(cursor):
-        current_data = find_node(identifier)(cursor)
-        if not current_data:
-            # no prior record exists, so regular insert
-            _insert_node(cursor, identifier, data)
-        else:
-            # merge the current and new data and update
-            updated_data = {**current_data, **data}
-            cursor.execute(read_sql(
-                'update-node.sql'), (json.dumps(_set_id(identifier, updated_data)), identifier,))
-    return _upsert_node
+    def _upsert(cursor):
+        _upsert_node(cursor, identifier, data)
+    return _upsert
+
+
+def upsert_nodes(nodes, ids):
+    def _upsert(cursor):
+        for (id, node) in zip(ids, nodes):
+            _upsert_node(cursor, id, node)
+    return _upsert
 
 
 def connect_nodes(source_id, target_id, properties={}):
@@ -158,7 +171,7 @@ def traverse(db_file, src, tgt=None, neighbors_fn=find_neighbors):
     def _traverse(cursor):
         path = []
         target = json.dumps(tgt)
-        for row in cursor.execute(neighbors_fn(), (json.dumps(src),)):
+        for row in cursor.execute(neighbors_fn(), {'source': src}):
             if row:
                 identifier = row[0]
                 if identifier not in path:
@@ -174,7 +187,7 @@ def traverse_with_bodies(db_file, src, tgt=None, neighbors_fn=find_neighbors):
         path = []
         target = json.dumps(tgt)
         header = None
-        for row in cursor.execute(neighbors_fn(True), (json.dumps(src),)):
+        for row in cursor.execute(neighbors_fn(True), {'source': src}):
             if not header:
                 header = row
                 continue
