@@ -1,5 +1,5 @@
 WITH RECURSIVE traverse(id) AS (
-  SELECT ?
+  SELECT :source
   UNION
   SELECT target FROM edges JOIN traverse ON source = id
 ) SELECT id FROM traverse;
